@@ -3,7 +3,12 @@
  * Author : Stephen Smalley, <sds@epoch.ncsc.mil> 
  */
 
-/* Updated: Karl MacMillan <kmacmillan@tresys.com>
+/*
+ * Updated: Trusted Computer Solutions, Inc. <dgoeddel@trustedcs.com>
+ *
+ *	Support for enhanced MLS infrastructure.
+ *
+ * Updated: Karl MacMillan <kmacmillan@tresys.com>
  *
  * 	Added conditional policy language extensions
  *
@@ -11,6 +16,7 @@
  *
  *	Added IPv6 support.
  *
+ * Copyright (C) 2004-2005 Trusted Computer Solutions, Inc.
  * Copyright (C) 2003 - 2004 Tresys Technology, LLC
  * Copyright (C) 2003 Red Hat, Inc., James Morris <jmorris@redhat.com>
  *	This program is free software; you can redistribute it and/or modify
@@ -88,10 +94,11 @@ char *txtfile = "policy.conf";
 char *binfile = "policy";
 
 unsigned int policyvers = POLICYDB_VERSION_MAX;
+unsigned int mlspol = 0;
 
 void usage(char *progname)
 {
-	printf("usage:  %s [-b] [-d] [-c policyvers (%d-%d)] [-o output_file] [input_file]\n",
+	printf("usage:  %s [-b] [-d] [-M] [-c policyvers (%d-%d)] [-o output_file] [input_file]\n",
 		progname, POLICYDB_VERSION_MIN, POLICYDB_VERSION_MAX);
 	exit(1);
 }
@@ -459,7 +466,7 @@ int main(int argc, char **argv)
 	struct policy_file pf;
 
 
-	while ((ch = getopt(argc, argv, "o:dbVc:")) != EOF) {
+	while ((ch = getopt(argc, argv, "o:dbMVc:")) != EOF) {
 		switch (ch) {
 		case 'o':
 			outfile = optarg;
@@ -474,6 +481,9 @@ int main(int argc, char **argv)
 		case 'V':
 			show_version = 1;
 			break;
+		case 'M':
+			mlspol = 1;
+			break;
 		case 'c': {
 			long int n = strtol(optarg, NULL, 10);
 			if (errno) {
@@ -497,11 +507,7 @@ int main(int argc, char **argv)
 	}
 
 	if (show_version) {
-#ifdef CONFIG_SECURITY_SELINUX_MLS	
-		printf("%d-mls (compatibility range %d-%d)\n", policyvers, POLICYDB_VERSION_MAX, POLICYDB_VERSION_MIN);
-#else
 		printf("%d (compatibility range %d-%d)\n", policyvers, POLICYDB_VERSION_MAX, POLICYDB_VERSION_MIN);
-#endif
 		exit(0);
 	}
 
@@ -546,6 +552,21 @@ int main(int argc, char **argv)
 			exit(1);
 		}
 		policydbp = &policydb;
+
+		/* Check Policy Consistency */
+		if (sepol_mls_enabled()) {
+			if (!mlspol) {
+				fprintf(stderr,"%s:  MLS policy, but non-MLS"
+				        " is specified\n", argv[0]);
+				exit(1);
+			}
+		} else {
+			if (mlspol) {
+				fprintf(stderr,"%s:  non-MLS policy, but MLS"
+				        " is specified\n", argv[0]);
+				exit(1);
+			}
+		}
 	} else {
 		yyin = fopen(file, "r");
 		if (!yyin) {
@@ -557,6 +578,9 @@ int main(int argc, char **argv)
 		if (policydb_init(&policydb))
 			exit(1);
 
+		/* Let sepol know if we are dealing with MLS support */
+		sepol_set_mls(mlspol);
+
 		id_queue = queue_create();
 		if (!id_queue) {
 			fprintf(stderr, "%s:  out of memory\n", argv[0]);
@@ -614,7 +638,12 @@ int main(int argc, char **argv)
 			exit(1);
 		}
 
-		sepol_set_policyvers(policyvers);
+		if (sepol_set_policyvers(policyvers)) {
+			fprintf(stderr,"%s:  incompatible policy (version %d) "
+				"for writing to %s\n", argv[0],
+				policyvers, outfile);
+			exit(1);
+		}
 
 		pf.type = PF_USE_STDIO;
 		pf.fp = outfp;
